import argparse

import typeguard

class ParseKwargs(argparse.Action):
    """
    Parse dictionary-like key-value pairs.

    References:
        * https://sumit-ghosh.com/posts/parsing-dictionary-key-value-pairs-kwargs-argparse-python/
    """
    @typeguard.typechecked
    def __call__(self, parser : argparse.ArgumentParser, namespace : argparse.Namespace, values : list, *args, **kwargs):
        setattr(namespace, self.dest, dict())
        for value in values:
            key, value = value.split('=', 2)
            getattr(namespace, self.dest)[key] = value



import argparse
import logging
import pathlib

import typeguard

from system_helpers.apt         import extract
from system_helpers.apt.install import install_packages, Cleaner

@typeguard.typechecked
def parse_args() -> argparse.Namespace:
    """
    Parse CLI arguments.
    """
    parser = argparse.ArgumentParser()

    subparsers = parser.add_subparsers(required = True)

    parser_ip = subparsers.add_parser('install-packages')

    parser_ip.add_argument('--clean',   action = 'store_true')
    parser_ip.add_argument('--update',  action = 'store_true')
    parser_ip.add_argument('--upgrade', action = 'store_true')

    parser_ip.add_argument(
        '--packages',
        help = "List of packages to install.",
        nargs = '*',
        required = False,
    )

    parser_ip.add_argument(
        '--requirement',
        help = 'Requirement file à la pip.',
        dest = 'requirements',
        action = 'append',
        type = pathlib.Path,
        required = False,
    )

    parser_ip.add_argument(
        '--args',
        help = 'Additional arguments to \'apt install\'.',
        nargs = argparse.REMAINDER,
        required = False,
    )

    parser_ip.set_defaults(func = install_packages)

    parser_ex = subparsers.add_parser('extract-from-package')

    parser_ex.add_argument('--package', required = True, type = str)
    parser_ex.add_argument('--files',   required = True, nargs = argparse.REMAINDER)

    parser_ex.set_defaults(func = extract.extract)

    parser_clean = subparsers.add_parser('clean')

    parser_clean.set_defaults(func = Cleaner.run)

    return parser.parse_args()

@typeguard.typechecked
def main() -> None:

    logging.basicConfig(level = logging.INFO)

    args = parse_args()

    kwargs = vars(args)

    func = kwargs.pop('func')

    func(**kwargs)

if __name__ == "__main__":

    main()



import pathlib
import subprocess
import typing

import typeguard

@typeguard.typechecked
def update_alternatives(*,
    for_each_of : typing.Dict[str, str],
    prefix : pathlib.Path,
    level : int,
    display : bool,
) -> None:
    """
    Update alternatives for a list of 'apps'.
    """
    for link, command in for_each_of.items():
        subprocess.check_call([
            'update-alternatives',
            '--install', prefix / link, link, prefix / command,
            str(level),
        ])

        if display:
            subprocess.check_call(['update-alternatives', '--display', link])



import argparse
import logging
import pathlib

import typeguard

from system_helpers.update_alternatives.alternatives     import update_alternatives
from system_helpers.update_alternatives.argparse_helpers import ParseKwargs

@typeguard.typechecked
def parse_args() -> argparse.Namespace:
    """
    Parse CLI arguments.
    """
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '--prefix',
        help = "Directory where alternatives are created.",
        required = False, type = pathlib.Path, default = pathlib.Path("/usr/bin"),
    )

    parser.add_argument(
        '--for-each-of',
        help = "List of link-command pairs.",
        nargs ='*',
        required = True,
        action = ParseKwargs,
    )

    parser.add_argument(
        '--level',
        help = "Priority level.",
        required = False, type = int, default = 10,
    )

    parser.add_argument(
        '--display',
        help = "Whether to display information.",
        required = False, action = 'store_true', default = True,
    )

    return parser.parse_args()

@typeguard.typechecked
def main() -> None:

    logging.basicConfig(level=logging.INFO)

    args = parse_args()

    update_alternatives(**vars(args))

if __name__ == "__main__":

    main()



import logging
import pathlib
import shutil
import subprocess
import typing

import typeguard

@typeguard.typechecked
def get_list_of_packages_from_requirements_file(*, file : pathlib.Path) -> typing.List[str]:
    """
    Get list of packages from a requirements file.

    Note that this function only supports skipping empty lines and comment lines, *i.e.*,
    not all features from https://pip.pypa.io/en/stable/reference/requirements-file-format/
    are supported.
    """
    packages = []
    with file.open(mode = "r") as file:
        for line in file:
            line = line.strip()
            if line and not line.startswith('#'):
                packages.extend(line.strip().split())
    return packages

@typeguard.typechecked
def install_command(*, yes : bool = True, no_install_recommends : bool = True) -> typing.List[str]:
    """
    Get the `apt` command to install packages.
    """
    cmd = ['apt']

    if yes:
        cmd.append('--yes')

    if no_install_recommends:
        cmd.append('--no-install-recommends')

    cmd.append('install')
    return cmd

class Cleaner:
    @typeguard.typechecked
    @staticmethod
    def run() -> None:
        logging.info("Cleaning 'apt' cache and lists.")
        subprocess.check_call(['apt', 'clean'])
        shutil.rmtree(pathlib.Path("/var/lib/apt/lists"))

@typeguard.typechecked
def install_packages(*,
    packages : typing.Optional[typing.List[str]] = None,
    requirements : typing.Optional[typing.List[pathlib.Path]] = None,
    update : bool = False,
    upgrade : bool = False,
    clean: bool = False,
    args : typing.Optional[typing.List[str]] = None,
) -> None:
    """
    Install list of packages by using `apt`.

    Optionally:
        * update
        * upgrade
        * clean
    """
    to_be_installed = []

    if packages:
        to_be_installed += packages

    if requirements:
        for file in requirements:
            to_be_installed += get_list_of_packages_from_requirements_file(file = file)

    if len(to_be_installed) == 0:
        raise RuntimeError('There is no package to be installed.')

    logging.info(f"Installing 'apt' packages {to_be_installed} (update={update}, upgrade={upgrade}, clean={clean}, args={args})")

    if update:
        subprocess.check_call(['apt', 'update'])

    if upgrade:
        subprocess.check_call(['apt', '--yes', 'upgrade'])

    cmd = install_command(yes = True, no_install_recommends = True)
    if args:
        cmd += args
    cmd += to_be_installed

    subprocess.check_call(cmd)

    if clean: Cleaner.run()



import argparse
import logging
import pathlib
import re
import subprocess
import tempfile
import typing
import urllib.parse

import typeguard

from system_helpers.apt import install

@typeguard.typechecked
def parse_args() -> argparse.Namespace:
    """
    Parse CLI arguments.
    """
    parser = argparse.ArgumentParser()

    parser.add_argument('--package', type = str, required = True)
    parser.add_argument('--files',   nargs='+')

    return parser.parse_args()

@typeguard.typechecked
def candidate(*, package : str, plus : bool) -> str:
    """
    Get candidate version for `package`.
    """
    output = subprocess.check_output(['apt', 'policy', package]).decode()
    try:
        version = re.search(
            pattern = r'Candidate: ([a-z0-9.\-~+:]+)',
            string  = output,
        ).group(1)
    except AttributeError:
        logging.exception(output)
        raise
    return urllib.parse.quote_plus(version).lower() if plus else version

@typeguard.typechecked
def extract(*, package : str, files : typing.List[str], arch : str = 'amd64', clean : bool = True) -> None:
    """
    Download `.deb` for `package` and install only the specified `files`.
    """
    logging.info(f'Extracting files {files} from package {package}.')

    install.install_packages(
        update = True,
        clean = False,
        args = ['--download-only'],
        packages = [package],
    )

    version_plus = candidate(package = package, plus = True)
    version_raw  = candidate(package = package, plus = False)

    logging.info(f'Candidate version for {package} is {version_plus} or {version_raw}.')

    deb = None

    for version in [version_plus, version_raw]:
        test_deb = pathlib.Path('/var/cache/apt/archives/') / f'{package}_{version}_{arch}.deb'

        if test_deb.is_file():
            deb = test_deb
            break

    if not deb:
        raise FileNotFoundError(deb)

    logging.info(f'The deb file is {deb}.')

    with tempfile.TemporaryDirectory() as tmpdir:
        # List files in the archive. We'll look for the data part.
        cmd = ['ar', 'tv', deb]
        content = re.search(
            pattern = r'(data\.[a-z\.]+)',
            string  = subprocess.check_output(cmd, cwd = tmpdir).decode(),
        ).group(1)

        cmd = ['ar', 'xv', deb, '--output=' + tmpdir]

        logging.info(f'Opening {deb} with {cmd}.')

        subprocess.check_call(cmd, cwd = tmpdir)

        data = pathlib.Path(tmpdir) / content

        if not data.is_file():
            raise FileNotFoundError(data)

        cmd = ['tar', '-C', '/', '-xvf', data, *files]

        logging.info(f'Extracting files with {cmd}.')

        subprocess.check_call(cmd, cwd = tmpdir)

    if clean: install.Cleaner.run()

if __name__ == '__main__':

    logging.basicConfig(level = logging.INFO)

    args = parse_args()

    logging.info(f"Received arguments: {args}")

    extract(**vars(args))
